@@ -1,13 +1,15 @@
+from __future__ import absolute_import
 import logging
 
-from .task import Task
-from .master import Master
-from .config import Config
+from kuyruk.task import Task
+from kuyruk.master import Master
+from kuyruk.config import Config
+from kuyruk.eventmixin import EventMixin
 
 logger = logging.getLogger(__name__)
 
 
-class Kuyruk(object):
+class Kuyruk(EventMixin):
     """
     Main class for Kuyruk distributed task queue. It holds the configuration
     values and provides a task decorator for user application and run method
@@ -20,9 +22,8 @@ class Kuyruk(object):
         :param config: See config.py for default values.
 
         """
+        super(Kuyruk, self).__init__()
         self.config = Config(config)
-        self.before_task_functions = []
-        self.after_task_functions = []
 
     def task(self, queue='kuyruk', eager=False, retry=0):
         """Wrap functions with this decorator to convert them to background
@@ -60,14 +61,6 @@ class Kuyruk(object):
         master.override_queues = queues
         master.run()
 
-    def before_task(self, f):
-        self.before_task_functions.append(f)
-        return f
-
-    def after_task(self, f):
-        self.after_task_functions.append(f)
-        return f
-
     class Reject(Exception):
         """
         The task should raise this if it does not want to process the message.
