@@ -7,42 +7,36 @@ import logging
 import itertools
 import multiprocessing
 from time import time, sleep
-
 from setproctitle import setproctitle
-
 from kuyruk.worker import Worker
+from kuyruk.process import Process
 
 logger = logging.getLogger(__name__)
 
 
-class Master(multiprocessing.Process):
+class Master(Process):
     """
     Master worker implementation that coordinates queue workers.
 
     """
     def __init__(self, config):
-        super(Master, self).__init__()
-        self.config = config
+        super(Master, self).__init__(config)
         self.workers = []
-        self.shutdown_pending = False
         self.override_queues = None
-
-    def run(self):
-        logger.debug('Process id: %s', os.getpid())
-        logger.debug('Process group id: %s', os.getpgrp())
-        setproctitle('kuyruk: master')
-        self._register_signals()
-
         if self.config.MAX_LOAD is None:
             self.config.MAX_LOAD = multiprocessing.cpu_count()
 
-        self._start_workers()
-        self._wait_for_workers()
+    def run(self):
+        super(Master, self).run()
+        setproctitle('kuyruk: master')
+        self.start_workers()
+        self.maybe_start_manager_thread()
+        self.wait_for_workers()
         logger.debug('End run master')
 
-    def _start_workers(self):
+    def start_workers(self):
         """Start a new worker for each queue"""
-        queues = self._get_queues()
+        queues = self.get_queues()
         queues = parse_queues_str(queues)
         logger.info('Starting to work on queues: %s', queues)
         for queue in queues:
@@ -50,7 +44,7 @@ class Master(multiprocessing.Process):
             worker.start()
             self.workers.append(worker)
 
-    def _get_queues(self):
+    def get_queues(self):
         """Return queues string."""
         if self.override_queues:
             return self.override_queues
@@ -71,7 +65,7 @@ class Master(multiprocessing.Process):
         for worker in workers:
             os.kill(worker.pid, signal.SIGKILL if kill else signal.SIGTERM)
 
-    def _wait_for_workers(self):
+    def wait_for_workers(self):
         """Loop until any of the self.workers is alive.
         If a worker is dead and Kuyruk is running state, spawn a new worker.
 
@@ -83,9 +77,9 @@ class Master(multiprocessing.Process):
             for worker in list(self.workers):
                 if worker.is_alive():
                     any_alive = True
-                elif not self.shutdown_pending:
-                    self._kill_pg(worker)
-                    self._spawn_new_worker(worker)
+                elif not self.shutdown_pending.is_set():
+                    worker.kill_pg()
+                    self.spawn_new_worker(worker)
                     any_alive = True
                 else:
                     self.workers.remove(worker)
@@ -95,7 +89,7 @@ class Master(multiprocessing.Process):
                              "%i seconds passed" % (time() - start))
                 sleep(1)
 
-    def _spawn_new_worker(self, worker):
+    def spawn_new_worker(self, worker):
         """Spawn a new process with parameters same as the old worker."""
         logger.debug("Spawning new worker")
         new_worker = Worker(worker.queue_name, self.config)
@@ -104,48 +98,61 @@ class Master(multiprocessing.Process):
         self.workers.remove(worker)
         logger.debug(self.workers)
 
-    def _kill_pg(self, worker):
-        try:
-            # Worker could spawn processes in task,
-            # kill them all.
-            os.killpg(worker.pid, signal.SIGKILL)
-        except OSError as e:
-            if e.errno != 3:  # No such process
-                raise
+    def register_signals(self):
+        signal.signal(signal.SIGINT, self.handle_sigint)
+        signal.signal(signal.SIGTERM, self.handle_sigterm)
+        signal.signal(signal.SIGHUP, self.handle_sighup)
+        signal.signal(signal.SIGQUIT, self.handle_sigquit)
 
-    def _register_signals(self):
-        signal.signal(signal.SIGINT, self._handle_sigint)
-        signal.signal(signal.SIGTERM, self._handle_sigterm)
-        signal.signal(signal.SIGHUP, self._handle_sighup)
-        signal.signal(signal.SIGQUIT, self._handle_sigquit)
+    def handle_sigterm(self, signum, frame):
+        logger.warning("Handling SIGTERM")
+        self.warm_shutdown()
 
-    def _handle_sigterm(self, signum, frame):
-        logger.warning("Warm shutdown")
-        self.shutdown_pending = True
-        self.stop_workers()
+    def handle_sigquit(self, signum, frame):
+        logger.warning("Handling SIGQUIT")
+        self.cold_shutdown()
 
-    def _handle_sigquit(self, signum, frame):
-        logger.warning("Cold shutdown")
-        self.shutdown_pending = True
-        self.stop_workers(kill=True)
-
-    def _handle_sigint(self, signum, frame):
+    def handle_sigint(self, signum, frame):
         logger.warning("Handling SIGINT")
-        if sys.stdin.isatty() and not self.shutdown_pending:
-            self._handle_sigterm(None, None)
+        if sys.stdin.isatty() and not self.shutdown_pending.is_set():
+            self.warm_shutdown()
         else:
-            self._handle_sigquit(None, None)
+            self.cold_shutdown()
         logger.debug("Handled SIGINT")
 
-    def _handle_sighup(self, signum, frame):
+    def handle_sighup(self, signum, frame):
         logger.warning("Handling SIGHUP")
+        self.reload()
+
+    def warm_shutdown(self):
+        logger.warning("Warm shutdown")
+        self.shutdown_pending.set()
+        self.stop_workers()
+
+    def cold_shutdown(self):
+        logger.warning("Cold shutdown")
+        self.shutdown_pending.set()
+        self.stop_workers(kill=True)
+
+    def reload(self):
+        logger.warning("Reloading workers")
         old_workers = self.workers
-        if self.config.path:
+        if hasattr(self.config, 'path'):
             self.config.reload()
             self.workers = []
-            self._start_workers()
+            self.start_workers()
         self.stop_workers(workers=old_workers)
 
+    def get_stats(self):
+        """Generate stats to be sent to manager."""
+        return {
+            'type': 'master',
+            'hostname': socket.gethostname(),
+            'uptime': self.uptime,
+            'pid': os.getpid(),
+            'load': os.getloadavg(),
+        }
+
 
 def parse_queues_str(s):
     """Parse command line queues string.
@@ -161,7 +168,10 @@ def parse_queues_str(s):
 def parse_count(q):
     parts = q.split('*', 1)
     if len(parts) > 1:
-        return int(parts[0]) * [parts[1]]
+        try:
+            return int(parts[0]) * [parts[1]]
+        except ValueError:
+            return int(parts[1]) * [parts[0]]
     return [parts[0]]
 
 
